<?php
/**
 * Anonymises a Paypal CSV file
 *
 * This work is licensed under the MIT License
 * Copyright (c) 2012 Bairwell Ltd
 * Permission is hereby granted, free of charge, to any person obtaining a copy of this
 * software and associated documentation files (the "Software"), to deal in the Software
 * without restriction, including without limitation the rights to use, copy, modify, merge,
 * publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons
 * to whom the Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all copies or
 * substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING
 * BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
 * DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 * PHP Version 5
 *
 * @package Bairwell
 * @subpackage Paypal\Reconciliation
 * @author Richard Bairwell
 * @copyright 2012 Bairwell Ltd - http://www.bairwell.com
 * @license MIT
 */
namespace Bairwell\Paypal;

class Anonymise
{

    private $name_maps;

    private $email_maps;

    private $transaction_id_maps;

    private $item_titles;

    /**
     * Process the file
     * @throws Reconciliation\Exception
     */
    public function process($input_file, $output_file)
    {
        if (is_string($input_file) === FALSE) {
            throw new Reconciliation\Exception('Input file is not a string/not set');
        }
        if (file_exists($input_file) === FALSE) {
            throw new Reconciliation\Exception('Input file does not exist');
        }
        if (is_readable($input_file) === FALSE) {
            throw new Reconciliation\Exception('Input file is not readable');
        }
        $input_handle = fopen($input_file, 'r');
        if ($input_handle === FALSE) {
            throw new Reconciliation\Exception('Unable to open ' . $input_file . ' for reading');
        }
        $output_handle = fopen($output_file, 'w');
        if ($output_handle === FALSE) {
            throw new Reconciliation\Exception('Unable to open ' . $output_file . ' for writing');
        }
        /**
         * Anonymise the data
         */
        $row_count = 0;
        while (($data = fgetcsv($input_handle, 2000, ',')) !== FALSE) {
            if ($row_count === 0) {
                $temp = implode(', ', $data);
                if ($temp !== Reconciliation\In::EXPECTED_HEADERS) {
                    throw new Reconciliation\Exception('Invalid header line');
                }
            } else {
                for ($col = 0; $col <= Reconciliation\In::COLUMN_BALANCE; $col++) {
                    switch ($col) {
                        case Reconciliation\In::COLUMN_NAME:
                            /** Names */
                            if (strlen($data[$col]) > 0) {
                                $hash = md5($data[$col]);
                                if (isset($this->name_maps[$hash]) === FALSE) {
                                    $this->name_maps[$hash] = 'Person ' . (count($this->name_maps) + 1);
                                }
                                $data[$col] = $this->name_maps[$hash];
                            }
                            break;
                        case Reconciliation\In::COLUMN_FROMEMAILADDRESS:
                        case Reconciliation\In::COLUMN_TOEMAILADDRESS:
                            /**
                             * Email addresses
                             */
                            if (strlen($data[$col]) > 0) {
                                $hash = md5($data[$col]);
                                if (isset($this->email_maps[$hash]) === FALSE) {
                                    $this->email_maps[$hash] = 'email_' . (count($this->email_maps) + 1) . '@example.com';
                                }
                                $data[$col] = $this->email_maps[$hash];
                            }
                            break;
                        case Reconciliation\In::COLUMN_TRANSACTIONID:
                        case Reconciliation\In::COLUMN_REFERENCETRANSACTIONID:
                            /**
                             * Transaction ids
                             */
                            if (strlen($data[$col]) > 0) {
                                $hash = md5($data[$col]);
                                if (isset($this->transaction_id_maps[$hash]) === FALSE) {
                                    $this->transaction_id_maps[$hash] = strtoupper(substr($hash . $hash . $hash, 0, strlen($data[$col])));
                                }
                                $data[$col] = $this->transaction_id_maps[$hash];
                            }
                            break;
                        case Reconciliation\In::COLUMN_INVOICENUMBER:
                        case Reconciliation\In::COLUMN_ITEMTITLE:
                            /**
                             * Item/invoice titles
                             */
                            if (strlen($data[$col]) > 0) {
                                $hash = md5($data[$col]);
                                if (isset($this->item_titles[$hash]) === FALSE) {
                                    $this->item_titles[$hash] = 'Item name ' . (count($this->item_titles) + 1);
                                }
                                $data[$col] = $this->item_titles[$hash];
                            }
                            break;
                    }
                }
            }
            fputcsv($output_handle, $data);
            $row_count++;
        }
        fclose($input_handle);
        fclose($output_handle);
    }
}