<?php
/**
 * Paypal Reconciliation
 *
 * Makes Paypal statements easier to reconcile in services such as Crunch accountancy
 *
 * This work is licensed under the MIT License
 * Copyright (c) 2012 Bairwell Ltd
 * Permission is hereby granted, free of charge, to any person obtaining a copy of this
 * software and associated documentation files (the "Software"), to deal in the Software
 * without restriction, including without limitation the rights to use, copy, modify, merge,
 * publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons
 * to whom the Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all copies or
 * substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING
 * BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
 * DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 * PHP Version 5
 *
 * @package Bairwell
 * @subpackage Paypal\Reconciliation
 * @author Richard Bairwell
 * @copyright 2012 Bairwell Ltd - http://www.bairwell.com
 * @license MIT
 */
namespace Bairwell\Paypal;

class Reconciliation
{

    /**
     * @var string The input filename
     */
    private $input_file;

    /**
     * @var string The base currency
     */
    private $base_currency;

    /**
     * @var Reconciliation\OutIterator Our Output data
     */
    private $outs;

    /**
     * @var Reconciliation\Balance The current balance
     */
    private $balance;


    public function __construct($arguments = array())
    {
        $this->base_currency = 'GBP';
        $this->input_file = NULL;
        $this->process_arguments($arguments);
    }

    /**
     * Process our arguments
     * @param array $arguments
     * @throws Reconciliation\Exception
     */
    private function process_arguments($arguments)
    {
        if (is_array($arguments) === TRUE) {
            if (isset($arguments['input_file']) === TRUE) {
                $this->input_file = $arguments['input_file'];
            }
            if (isset($arguments['base_currency']) === TRUE) {
                if (in_array($arguments['base_currency'], array('GBP', 'EUR', 'JPN', 'USD')) == FALSE) {
                    throw new Reconciliation\Exception('Unrecognised currency');
                }
                $this->base_currency = $arguments['base_currency'];
            }
        }
    }


    /**
     * Process the file
     * @throws Reconciliation\Exception
     */
    public function process($arguments = array())
    {
        $this->process_arguments($arguments);
        if (is_string($this->input_file) === FALSE) {
            throw new Reconciliation\Exception('Input file is not a string/not set');
        }
        if (file_exists($this->input_file) === FALSE) {
            throw new Reconciliation\Exception('Input file does not exist');
        }
        if (is_readable($this->input_file) === FALSE) {
            throw new Reconciliation\Exception('Input file is not readable');
        }
        $input_handle = fopen($this->input_file, 'r');
        if ($input_handle === FALSE) {
            throw new Reconciliation\Exception('Unable to open ' . $this->input_file . ' for reading');
        }
        /**
         * First cache the currency conversions
         */
        $currency_conversions = array();
        $row_count = 0;
        while (($data = fgetcsv($input_handle, 2000, ',')) !== FALSE) {
            if ($row_count === 0) {
                $temp = implode(', ', $data);
                if ($temp !== Reconciliation\In::EXPECTED_HEADERS) {
                    throw new Reconciliation\Exception('Invalid header line');
                }
            } else {
                $line = new Reconciliation\In($data, array());
                if ($line->get_currency() !== $this->base_currency && $line->get_type() !== 'Currency Conversion') {
                    $new_convert = new Reconciliation\Currency();
                    $new_convert->set_originalCurrency($line->get_currency());
                    $new_convert->set_originalAmount($line->get_gross());
                    $new_convert->set_originalTransactionId($line->get_transactionId());
                    $currency_conversions[$line->get_transactionId()] = $new_convert;
                } elseif ($line->get_currency() === $this->base_currency && $line->get_type() === 'Currency Conversion') {
                    if (isset($currency_conversions[$line->get_relatedTransaction()]) === TRUE) {
                        /**
                         * @var Reconciliation\Currency $convert
                         */
                        $convert = $currency_conversions[$line->get_relatedTransaction()];
                        $convert->set_baseAmount($line->get_gross());
                        $convert->set_baseCurrency($this->base_currency);
                        $convert->set_baseTransactionId($line->get_transactionId());
                        $convert->set_baseBalance($line->get_balance());
                        $currency_conversions[$line->get_relatedTransaction()] = $convert;
                    } else {
                        throw new Reconciliation\Exception('Unable to find matching transaction for currency conversion ' . $line->get_transactionId());
                    }
                }
            }
            $row_count++;
        }
        rewind($input_handle);
        fgetcsv($input_handle, 2000, ','); // skip the first line
        /**
         * Now actually extract the data
         */
        $this->outs = new Reconciliation\OutIterator();
        $this->balance = new Reconciliation\Balance();
        while (($data = fgetcsv($input_handle, 2000, ',')) !== FALSE) {
            $line = new Reconciliation\In($data, $currency_conversions);
            $out = NULL;
            switch ($line->get_type()) {
                case 'Cancelled Fee':
                    break;
                case 'Request Sent':
                    break;
                case 'Invoice Sent':
                    break;
                case 'Currency Conversion':
                    break;
                case 'Shopping Cart Item':
                    throw new Reconciliation\Exception('Sorry, can not cope with " Include Shopping Cart details"');
                    break;
                case 'Refund':
                    $out = new Reconciliation\Out($line);
                    $out->set_description('Refund: ' . $line->get_name());
                    $out->set_amount(0 - $line->get_net());
                    $this->outs->push($out);
                    if ($line->get_fee() + $line->get_net() !== $line->get_gross()) {
                        throw new Reconciliation\Exception($line->get_type() . ' gross does not equal fee+net:' . $line->get_transactionId());
                    }
                    $this->balance->process($line, 0, $line->get_net());
                    break;
                case 'Shopping Cart Payment Received':
                    $out = new Reconciliation\Out($line);
                    $out->set_description('Shopping cart: ' . $line->get_name() . ' : ' . $line->get_invoiceNumber());
                    $out->set_amount($line->get_gross());
                    $this->outs->push($out);
                    $out = new Reconciliation\Out($line);
                    $out->set_description('To/From Paypal: Fee Paid: ' . $line->get_name() . ' : ' . $line->get_invoiceNumber());
                    $out->set_amount(0 - $line->get_fee());
                    $this->outs->push($out);
                    $expected = round($line->get_gross() - $line->get_fee(), 2);
                    if ($expected !== $line->get_net()) {
                        throw new Reconciliation\Exception($line->get_type() . ' net (' . $line->get_net() . ') does not equal gross (' . $line->get_gross() . ') -fee (' . $line->get_fee() . ') ' . $expected . ': ' . $line->get_transactionId());
                    }
                    $this->balance->process($line, $line->get_gross(), $line->get_fee());
                    break;
                case 'Donation Received':
                    $out = new Reconciliation\Out($line);
                    $out->set_description('Donation: ' . $line->get_name());
                    $out->set_amount($line->get_gross());
                    $this->outs->push($out);
                    $out = new Reconciliation\Out($line);
                    $out->set_description('To/From Paypal: Fee Paid: ' . $line->get_name());
                    $out->set_amount(0 - $line->get_fee());
                    $this->outs->push($out);
                    if ($line->get_gross() - $line->get_fee() !== $line->get_net()) {
                        throw new Reconciliation\Exception($line->get_type() . ' net does not equal gross-fee:' . $line->get_transactionId());
                    }
                    $this->balance->process($line, $line->get_gross(), $line->get_fee());
                    break;
                case 'Payment Received':
                    $out = new Reconciliation\Out($line);
                    $out->set_description('Payment: ' . $line->get_name());
                    $out->set_amount($line->get_gross());
                    $this->outs->push($out);
                    $out = new Reconciliation\Out($line);
                    $out->set_description('To/From Paypal: Fee Paid: ' . $line->get_name());
                    $out->set_amount(0 - $line->get_fee());
                    $this->outs->push($out);
                    if ($line->get_gross() - $line->get_fee() !== $line->get_net()) {
                        throw new Reconciliation\Exception($line->get_type() . ' net does not equal gross-fee:' . $line->get_transactionId());
                    }
                    $this->balance->process($line, $line->get_gross(), $line->get_fee());
                    break;
                case 'Withdraw Funds to a Bank Account':
                    $out = new Reconciliation\Out($line);
                    $out->set_description('To Bank Account');
                    $out->set_amount(0 - $line->get_net());
                    if ($line->get_net() !== $line->get_gross()) {
                        throw new Reconciliation\Exception('Invalid bank account line: ' . $line->get_date());
                    }
                    $this->outs->push($out);
                    $this->balance->process($line, 0, $line->get_gross());
                    break;
                case 'Subscription Payment Sent':
                    $out = new Reconciliation\Out($line);
                    $out->set_description('Subscription Sent: ' . $line->get_name());
                    $out->set_amount(0 - $line->get_net());
                    $this->outs->push($out);
                    $this->balance->process($line, 0, $line->get_net());
                    break;
                default:
                    throw new Reconciliation\Exception('Unrecognised transaction type "' . $line->get_type() . '" for transaction ' . $line->get_transactionId());
            }
        }
        fclose($input_handle);
    }

    /**
     * @return \Bairwell\Paypal\Reconciliation\Balance
     */
    public function get_balance()
    {
        return $this->balance;
    }

    /**
     * @return \Bairwell\Paypal\Reconciliation\OutIterator
     */
    public function get_outs()
    {
        return $this->outs;
    }

    /**
     * @return string
     */
    public function get_base_currency()
    {
        return $this->base_currency;
    }

    /**
     * @return string
     */
    public function get_input_file()
    {
        return $this->input_file;
    }


}



