<?php
/**
 * Handle a Paypal input line
 *
 * This work is licensed under the MIT License
 * Copyright (c) 2012 Bairwell Ltd
 * Permission is hereby granted, free of charge, to any person obtaining a copy of this
 * software and associated documentation files (the "Software"), to deal in the Software
 * without restriction, including without limitation the rights to use, copy, modify, merge,
 * publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons
 * to whom the Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all copies or
 * substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING
 * BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
 * DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 * PHP Version 5
 *
 * @package Bairwell
 * @subpackage Paypal\Reconciliation
 * @author Richard Bairwell
 * @copyright 2012 Bairwell Ltd - http://www.bairwell.com
 * @license MIT
 */
namespace Bairwell\Paypal\Reconciliation;


/**
 * Handle a Paypal CSV line
 */
class In
{
    /**
     * @var string The headers of the file we are expecting
     */
    const EXPECTED_HEADERS = 'Date, Time, Time Zone, Name, Type, Status, Currency, Gross, Fee, Net, From Email Address, To Email Address, Transaction ID, Address Status, Item Title, Item ID, Reference Txn ID, Invoice Number, Receipt ID, Balance, ';

    const COLUMN_DATE = 0;
    const COLUMN_TIME = 1;
    const COLUMN_TIMEZONE = 2;
    const COLUMN_NAME = 3;
    const COLUMN_TYPE = 4;
    const COLUMN_STATUS = 5;
    const COLUMN_CURRENCY = 6;
    const COLUMN_GROSS = 7;
    const COLUMN_FEE = 8;
    const COLUMN_NET = 9;
    const COLUMN_FROMEMAILADDRESS = 10;
    const COLUMN_TOEMAILADDRESS = 11;
    const COLUMN_TRANSACTIONID = 12;
    const COLUMN_ADDRESSSTATUS = 13;
    const COLUMN_ITEMTITLE = 14;
    const COLUMN_ITEMID = 15;
    const COLUMN_REFERENCETRANSACTIONID = 16;
    const COLUMN_INVOICENUMBER = 17;
    const COLUMN_RECEIPTID = 18;
    const COLUMN_BALANCE = 19;

    /**
     * @var string The date as sent by Paypal
     */
    private $date;
    /**
     * @var string The time as sent by Paypal
     */
    private $time;
    /**
     * @var string The timezone as sent by Payapl
     */
    private $timezone;
    /**
     * @var string The name of the payee/receiver
     */
    private $name;
    /**
     * @var string The type of the transaction
     */
    private $type;
    /**
     * @var string Status of the transaction
     */
    private $status;
    /**
     * @var string Currency of this transaction
     */
    private $currency;
    /**
     * @var float Gross amount (i.e. fee+net)
     */
    private $gross;
    /**
     * @var float The Paypal fee they've deducted/refunded
     */
    private $fee;
    /**
     * @var float The net amount paid
     */
    private $net;
    /**
     * @var string Who was it sent from
     */
    private $fromEmail;
    /**
     * @var string The transaction id
     */
    private $transactionId;
    /**
     * @var string Any related transaction ids (useful for currency conversions and refunds)
     */
    private $relatedTransaction;
    /**
     * @var string Any invoice numbers/strings
     */
    private $invoiceNumber;
    /**
     * @var float The current balance in this currency
     */
    private $balance;
    /**
     * @var string Notes about any currency conversions
     */
    private $conversionNotes;

    /**
     * Constructor
     * @param array $data The array data from the CSV file
     * @param array $convert Any currency conversion data we should use
     * @throws Exception If anything is deemed as invalid
     **/
    public function __construct($data, $convert = array())
    {
        $this->date = $data[self::COLUMN_DATE];
        if (preg_match('/^(0[1-9]|[1-2][0-9]|3[0-1])\/(0[1-9]|1[0-2])\/(199[0-9]|20[0-2][0-9])$/', $this->date) !== 1) {
            throw new Exception('Invalid date: ' . $this->date);
        }
        $this->time = $data[self::COLUMN_TIME];
        if (preg_match('/^(0[0-9]|1[0-9]|2[0-3]):[0-5][0-9]:[0-5][0-9]$/', $this->time) !== 1) {
            throw new Exception('Invalid time: ' . $this->time);
        }
        $this->timezone = $data[self::COLUMN_TIMEZONE];
        $this->name = $data[self::COLUMN_NAME];
        $this->type = $data[self::COLUMN_TYPE];
        $this->status = $data[self::COLUMN_STATUS];
        $this->currency = $data[self::COLUMN_CURRENCY];
        if (in_array($this->currency, array('GBP', 'EUR', 'JPN', 'USD')) === FALSE) {
            throw new Exception('Invalid currency: ' . $this->currency);
        }
        if (preg_match('/^\-?[0-9]{1,}(\.[0-9][0-9])?$/', $data[self::COLUMN_GROSS]) !== 1) {
            throw new Exception('Invalid gross figure: ' . $data[self::COLUMN_GROSS]);
        } else {
            $this->gross = abs($data[self::COLUMN_GROSS]);
        }
        if (preg_match('/^\-?[0-9]{1,}(\.[0-9][0-9])?$/', $data[self::COLUMN_FEE]) !== 1) {
            throw new Exception('Invalid fee figure: ' . $data[self::COLUMN_FEE]);
        } else {
            $this->fee = abs($data[self::COLUMN_FEE]);
        }
        if (preg_match('/^\-?[0-9]{1,}(\.[0-9][0-9])?$/', $data[self::COLUMN_NET]) !== 1) {
            throw new Exception('Invalid net figure: ' . $data[self::COLUMN_NET]);
        } else {
            $this->net = abs($data[self::COLUMN_NET]);
        }
        $this->fromEmail = $data[self::COLUMN_FROMEMAILADDRESS];
        $this->transactionId = $data[self::COLUMN_TRANSACTIONID];
        $this->relatedTransaction = $data[self::COLUMN_REFERENCETRANSACTIONID];
        $this->invoiceNumber = $data[self::COLUMN_INVOICENUMBER];

        if (preg_match('/^\-?[0-9]{1,}(\.[0-9][0-9])?$/', $data[self::COLUMN_BALANCE]) !== 1) {
            throw new Exception('Invalid balance figure: ' . $data[self::COLUMN_BALANCE]);
        } else {
            $this->balance = $data[self::COLUMN_BALANCE];
        }
        if (isset($convert[$this->transactionId]) === TRUE) {
            /**
             * @var Currency $converted
             */
            $converted = $convert[$this->get_transactionId()];
            $this->net = $converted->get_baseAmount();
            $this->balance = $converted->get_baseBalance();
            $this->conversionNotes = $converted->get_baseCurrency();
            $this->conversionNotes = 'Converted from ' . $this->get_net() . ' ' . $this->get_currency();
        } else {
            $this->conversionNotes = '';
        }
    }

    /**
     * Gets the date/time of this transaction
     * @return \DateTime
     * @throws Exception If unable to create a PHP \DateTime object from Paypal data
     */
    public function get_datetime()
    {
        $time = \DateTime::createFromFormat('d/m/Y G:i:s', $this->date . ' ' . $this->time);
        if ($time === FALSE) {
            throw new Exception('Unable to decode time ' . $this->date . ' ' . $this->time . ': ' . print_r(\DateTime::getLastErrors(), TRUE));
        }
        $time->setTimezone(new \DateTimeZone($this->timezone));
        return $time;
    }

    /**
     * Gets the related transaction id
     * @return string
     */
    public function get_relatedTransaction()
    {
        return $this->relatedTransaction;
    }

    /**
     * Gets the transaction id
     * @return string
     */
    public function get_transactionId()
    {
        return $this->transactionId;
    }


    /**
     * Gets the balance
     * @return float
     */
    public function get_balance()
    {
        return $this->balance;
    }

    /**
     * Gets the currency
     * @return string
     */
    public function get_currency()
    {
        return $this->currency;
    }

    /**
     * Gets the date
     * @return string
     */
    public function get_date()
    {
        return $this->date;
    }

    /**
     * Gets the fee
     * @return float
     */
    public function get_fee()
    {
        return round($this->fee, 2);
    }

    /**
     * Gets the from email
     * @return string
     */
    public function get_fromEmail()
    {
        return $this->fromEmail;
    }

    /**
     * Gets the invoice number
     * @return string
     */
    public function get_invoiceNumber()
    {
        return $this->invoiceNumber;
    }


    /**
     * Gets the gross figure
     * @return float
     */
    public function get_gross()
    {
        return round($this->gross, 2);
    }

    /**
     * Gets the name
     * @return string
     */
    public function get_name()
    {
        return $this->name;
    }

    /**
     * Gets the net amount
     * @return float
     */
    public function get_net()
    {
        return round($this->net, 2);
    }

    /**
     * Gets the transaction status
     * @return string
     */
    public function get_status()
    {
        return $this->status;
    }

    /**
     * Gets the time
     * @return string
     */
    public function get_time()
    {
        return $this->time;
    }

    /**
     * Gets the timezone
     * @return string
     */
    public function get_timezone()
    {
        return $this->timezone;
    }

    /**
     * Gets the transaction type
     * @return string
     */
    public function get_type()
    {
        return $this->type;
    }


}